﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Win32;

namespace TweakAPI.Info
{
    /// <summary>
    /// Provides information about the installed applications.
    /// </summary>
    public struct Program
    {
        /// <summary>
        /// Returns a <see cref="Program"/>-Array containing information about the installed applications.
        /// </summary>
        /// <returns></returns>
        public static Program[] EnumeratePrograms()
        {
            List<Program> programs = new List<Program>();
            RegistryKey mainKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall", 
                RegistryKeyPermissionCheck.ReadSubTree);
            foreach (string subKeyName in mainKey.GetSubKeyNames())
            {
                RegistryKey subKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\" + 
                    subKeyName, RegistryKeyPermissionCheck.ReadSubTree);
                if (subKey.GetValue("DisplayName") != null)
                {
                    Program prog = new Program();
                    prog.Name = subKey.GetValue("DisplayName").ToString();

                    try { prog.Publisher = subKey.GetValue("Publisher").ToString(); }
                    catch { prog.Publisher = ""; }

                    try { prog.Size = Math.Round(Convert.ToDecimal(subKey.GetValue("EstimatedSize").ToString()) / 1024, 1); }
                    catch { prog.Size = 0; }

                    try { prog.Version = subKey.GetValue("DisplayVersion").ToString(); }
                    catch { prog.Version = ""; }

                    try { prog.UninstallPath = subKey.GetValue("UninstallString").ToString(); }
                    catch { }

                    programs.Add(prog);
                }
            }
            RegistryKey mainKey2 = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Wow6432Node\\Microsoft\\Windows\\CurrentVersion" + 
                "\\Uninstall", RegistryKeyPermissionCheck.ReadSubTree);
            foreach (string subKeyName in mainKey2.GetSubKeyNames())
            {
                RegistryKey subKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Wow6432Node\\Microsoft\\Windows\\CurrentVersion" + 
                    "\\Uninstall\\" + subKeyName, RegistryKeyPermissionCheck.ReadSubTree);
                if (subKey.GetValue("DisplayName") != null)
                {
                    Program prog = new Program();
                    prog.Name = subKey.GetValue("DisplayName").ToString();

                    try { prog.Publisher = subKey.GetValue("Publisher").ToString(); }
                    catch { prog.Publisher = ""; }

                    try { prog.Size = Math.Round(Convert.ToDecimal(subKey.GetValue("EstimatedSize").ToString()) / 1024, 1); }
                    catch { prog.Size = 0; }

                    try { prog.Version = subKey.GetValue("DisplayVersion").ToString(); }
                    catch { prog.Version = ""; }

                    try { prog.UninstallPath = subKey.GetValue("UninstallString").ToString(); }
                    catch { }

                    programs.Add(prog);
                }
            }
            RegistryKey mainKey3 = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall", 
                RegistryKeyPermissionCheck.ReadSubTree);
            foreach (string subKeyName in mainKey3.GetSubKeyNames())
            {
                RegistryKey subKey = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\" + 
                    subKeyName, RegistryKeyPermissionCheck.ReadSubTree);
                if (subKey.GetValue("DisplayName") != null)
                {
                    Program prog = new Program();
                    prog.Name = subKey.GetValue("DisplayName").ToString();

                    try { prog.Publisher = subKey.GetValue("Publisher").ToString(); }
                    catch { prog.Publisher = ""; }

                    try { prog.Size = Math.Round(Convert.ToDecimal(subKey.GetValue("EstimatedSize").ToString()) / 1024, 1); }
                    catch { prog.Size = 0; }

                    try { prog.Version = subKey.GetValue("DisplayVersion").ToString(); }
                    catch { prog.Version = ""; }

                    try { prog.UninstallPath = subKey.GetValue("UninstallString").ToString(); }
                    catch { }

                    programs.Add(prog);
                }
            }

            return programs.Distinct().ToArray<Program>();
        }

        /// <summary>
        /// The name.
        /// </summary>
        public string Name { get; set; }
        /// <summary>
        /// The publisher.
        /// </summary>
        public string Publisher { get; set; }
        /// <summary>
        /// The size.
        /// </summary>
        public decimal Size { get; set; }
        /// <summary>
        /// The version.
        /// </summary>
        public string Version { get; set; }
        /// <summary>
        /// The path for the uninstall routine.
        /// </summary>
        public string UninstallPath { get; set; }
    }
}
